/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.callcontext;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.user.UserRole;

class CurrentCallContextUserServiceTest {

	@Spy
	private CurrentCallContextUserService userService;

	private final CallContextUser callContextUser = CallContextUserTestFactory.create();

	@DisplayName("Clear call context user")
	@Nested
	class TestClearCallContextUser {

		@Test
		void shouldSetUserToNull() {
			userService.clearCallContextUser();

			assertThat(userService.getCurrentCallContextUser()).isEmpty();
		}
	}

	@Nested
	class TestGetCurrentCallContextUser {

		@BeforeEach
		void init() {
			userService.setCallContextUser(callContextUser);
		}

		@Test
		void shouldGetUser() {
			assertThat(userService.getCurrentCallContextUser()).isPresent().get().isEqualTo(callContextUser);
		}
	}

	@Nested
	class TestHasRole {

		private final CallContextUser callContextUser = CallContextUserTestFactory.createBuilder()
				.roles(List.of(UserRole.VERWALTUNG_LOESCHEN))
				.build();

		@BeforeEach
		void init() {
			userService.setCallContextUser(callContextUser);
		}

		@Test
		void shouldCheckRoleWithinHierarchy() {
			try (var mock = mockStatic(UserRole.class)) {
				mock.when(() -> UserRole.containsWithinRoleHierarchy(callContextUser.getRoles(), UserRole.VERWALTUNG_USER)).thenReturn(true);

				userService.hasRole(UserRole.VERWALTUNG_USER);

				mock.verify(() -> UserRole.containsWithinRoleHierarchy(callContextUser.getRoles(), UserRole.VERWALTUNG_USER));
			}
		}

		@Test
		void shouldReturnTrue() {
			var hasRole = userService.hasRole(UserRole.VERWALTUNG_USER);

			assertThat(hasRole).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var hasRole = userService.hasRole("unknown");

			assertThat(hasRole).isFalse();
		}

	}
}
