/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.callcontext;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;

import jakarta.ws.rs.core.MultivaluedHashMap;
import jakarta.ws.rs.ext.ReaderInterceptorContext;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Spy;


class HttpRequestInterceptorTest {
	@Spy
	private HttpRequestInterceptor interceptor;

	@Nested
	class TestInterceptor {

		@Mock
		private ReaderInterceptorContext context;

		@Nested
		class WithRequestId {
			String id = UUID.randomUUID().toString();

			@BeforeEach
			void initHeader() {
				var headers = new MultivaluedHashMap<String, String>();

				headers.put(HttpRequestInterceptor.REQUEST_ID_KEY, List.of(id));
				when(context.getHeaders()).thenReturn(headers);
			}

			@Test
			void shouldSetRequestId() {
				var requestId = interceptor.getRequestId(context);

				assertThat(requestId).isEqualTo(id);
			}
		}

		@Nested
		class WithoutRequestId {

			@BeforeEach
			void initHeader() {
				var headers = new MultivaluedHashMap<String, String>();
				headers.put("xxx", List.of("y"));
				when(context.getHeaders()).thenReturn(headers);
			}

			@Test
			void shouldSetDummyRequestId() {
				var requestId = interceptor.getRequestId(context);

				assertThat(requestId).isEqualTo(HttpRequestInterceptor.REQUEST_ID_PLACEHOLDER);
			}
		}

	}
}
